Hello {{ user.full_name|default:user }},

We received a request to reset your {{ project_title }} password.
Click the link below to choose a new one:

{{ project_website }}{% url 'authentication:password_reset_confirm_view' uid token %}
