from django.utils.translation import ugettext_lazy as _

DEFAULT_EMAIL_IMAP_MAILBOX = 'INBOX'
DEFAULT_EMAIL_IMAP_SEARCH_CRITERIA = 'NOT DELETED'
DEFAULT_EMAIL_IMAP_STORE_COMMANDS = '+FLAGS (\\Deleted)'
DEFAULT_EMAIL_METADATA_ATTACHMENT_NAME = 'metadata.yaml'
DEFAULT_EMAIL_POP3_TIMEOUT = 60
DEFAULT_PERIOD_INTERVAL = 600

SOURCE_UNCOMPRESS_CHOICE_ALWAYS = 'y'
SOURCE_UNCOMPRESS_CHOICE_NEVER = 'n'
SOURCE_UNCOMPRESS_CHOICE_ASK = 'a'

SOURCE_INTERVAL_UNCOMPRESS_CHOICES = (
    (SOURCE_UNCOMPRESS_CHOICE_ALWAYS, _('Always')),
    (SOURCE_UNCOMPRESS_CHOICE_NEVER, _('Never')),
)

SOURCE_INTERACTIVE_UNCOMPRESS_CHOICES = (
    (SOURCE_UNCOMPRESS_CHOICE_ALWAYS, _('Always')),
    (SOURCE_UNCOMPRESS_CHOICE_NEVER, _('Never')),
    (SOURCE_UNCOMPRESS_CHOICE_ASK, _('Ask user'))
)
