{% load static %}

<link href="{% static 'sources/node_modules/dropzone/dist/dropzone.css' %}" media="screen" rel="stylesheet" type="text/css" />
<link href="{% static 'sources/css/sources.css' %}" media="screen" rel="stylesheet" type="text/css" />
